@@ -3,6 +3,7 @@
 
 #include <inttypes.h>
 #include <string>
+#include <vector>
 #include <openssl/bn.h>
 #include <openssl/dh.h>
 #include <openssl/aes.h>
@@ -13,11 +14,29 @@
 #include <openssl/evp.h>
 #include <openssl/rand.h>
 #include <openssl/engine.h>
+#include <openssl/opensslv.h>
 
 #include "Base.h"
 #include "Tag.h"
 #include "CPU.h"
 
+// recognize openssl version and features
+#if ((OPENSSL_VERSION_NUMBER < 0x010100000) || defined(LIBRESSL_VERSION_NUMBER)) // 1.0.2 and below or LibreSSL
+#   define LEGACY_OPENSSL 1
+#   define X509_getm_notBefore X509_get_notBefore
+#   define X509_getm_notAfter X509_get_notAfter
+#else
+#   define LEGACY_OPENSSL 0
+#   if (OPENSSL_VERSION_NUMBER >= 0x010101000) // 1.1.1
+#	   define OPENSSL_EDDSA 1
+#	   define OPENSSL_X25519 1
+#	   define OPENSSL_SIPHASH 1
+#   endif
+#   if !defined OPENSSL_NO_CHACHA && !defined OPENSSL_NO_POLY1305 // some builds might not include them
+#	   define OPENSSL_AEAD_CHACHA20_POLY1305 1 
+#   endif
+#endif
+
 namespace i2p
 {
 namespace crypto
@@ -48,6 +67,32 @@ namespace crypto
 			uint8_t m_PublicKey[256];
 	};
 
+	// x25519
+	class X25519Keys
+	{
+		public:
+
+			X25519Keys ();
+			X25519Keys (const uint8_t * priv, const uint8_t * pub); // for RouterContext
+			~X25519Keys ();
+
+			void GenerateKeys ();
+			const uint8_t * GetPublicKey () const { return m_PublicKey; };
+			void GetPrivateKey (uint8_t * priv) const;
+			void Agree (const uint8_t * pub, uint8_t * shared);			
+
+		private:
+
+			uint8_t m_PublicKey[32];		
+#if OPENSSL_X25519
+			EVP_PKEY_CTX * m_Ctx;
+			EVP_PKEY * m_Pkey;
+#else			
+			BN_CTX * m_Ctx;
+			uint8_t m_PrivateKey[32];
+#endif			
+	};
+	
 	// ElGamal
 	void ElGamalEncrypt (const uint8_t * key, const uint8_t * data, uint8_t * encrypted, BN_CTX * ctx, bool zeroPadding = false);
 	bool ElGamalDecrypt (const uint8_t * key, const uint8_t * encrypted, uint8_t * data, BN_CTX * ctx, bool zeroPadding = false);
@@ -69,30 +114,16 @@ namespace crypto
 
 		void operator^=(const ChipherBlock& other) // XOR
 		{
-			if (i2p::cpu::avx)
-			{
-#ifdef AVX
-				__asm__
-					(
-						"vmovups (%[buf]), %%xmm0 \n"
-						"vmovups (%[other]), %%xmm1 \n"
-						"vxorps %%xmm0, %%xmm1, %%xmm0 \n"
-						"vmovups %%xmm0, (%[buf]) \n"
-						:
-						: [buf]"r"(buf), [other]"r"(other.buf)
-						: "%xmm0", "%xmm1", "memory"
-						);
-#else
-				for (int i = 0; i < 16; i++)
-					buf[i] ^= other.buf[i];
-#endif
-			}
+			if (!(((size_t)buf | (size_t)other.buf) & 0x03)) // multiple of 4 ?
+			{	
+				for (int i = 0; i < 4; i++)
+					reinterpret_cast<uint32_t *>(buf)[i] ^= reinterpret_cast<const uint32_t *>(other.buf)[i];
+			}	
 			else
-			{
-				// TODO: implement it better
+			{	
 				for (int i = 0; i < 16; i++)
 					buf[i] ^= other.buf[i];
-			}
+			}					
 		}
 	};
 
@@ -123,7 +154,7 @@ namespace crypto
 	};
 
 
-#ifdef AESNI
+#ifdef __AES__
 	#ifdef ARM64AES
 		void init_aesenc(void) __attribute__((constructor));
 	#endif
@@ -143,7 +174,7 @@ namespace crypto
 	};
 #endif
 
-#ifdef AESNI
+#ifdef __AES__
 	class ECBEncryption: public ECBCryptoAESNI
 #else
 	class ECBEncryption
@@ -152,14 +183,14 @@ namespace crypto
 		public:
 
 		void SetKey (const AESKey& key);
-		
+
 		void Encrypt(const ChipherBlock * in, ChipherBlock * out);
 
 	private:
 		AES_KEY m_Key;
 	};
 
-#ifdef AESNI
+#ifdef __AES__
 	class ECBDecryption: public ECBCryptoAESNI
 #else
 	class ECBDecryption
@@ -188,7 +219,7 @@ namespace crypto
 			void Encrypt (const uint8_t * in, uint8_t * out); // one block
 
 			ECBEncryption & ECB() { return m_ECBEncryption; }
-		
+
 		private:
 
 			AESAlignedBuffer<16> m_LastBlock;
@@ -211,7 +242,7 @@ namespace crypto
 			void Decrypt (const uint8_t * in, uint8_t * out); // one block
 
 			ECBDecryption & ECB() { return m_ECBDecryption; }
-		
+
 		private:
 
 			AESAlignedBuffer<16> m_IV;
@@ -255,17 +286,17 @@ namespace crypto
 	};
 
 // AEAD/ChaCha20/Poly1305
-	bool AEADChaCha20Poly1305 (const uint8_t * msg, size_t msgLen, const uint8_t * ad, size_t adLen, const uint8_t * key, const uint8_t * nonce, uint8_t * buf, size_t len, bool encrypt); // msgLen is len without tag 
-   	
+	bool AEADChaCha20Poly1305 (const uint8_t * msg, size_t msgLen, const uint8_t * ad, size_t adLen, const uint8_t * key, const uint8_t * nonce, uint8_t * buf, size_t len, bool encrypt); // msgLen is len without tag
+
+	void AEADChaCha20Poly1305Encrypt (const std::vector<std::pair<uint8_t *, size_t> >& bufs, const uint8_t * key, const uint8_t * nonce, uint8_t * mac); // encrypt multiple buffers with zero ad
+
 // init and terminate
 	void InitCrypto (bool precomputation);
 	void TerminateCrypto ();
 }
 }
 
-// take care about openssl version
-#include <openssl/opensslv.h>
-#define LEGACY_OPENSSL ((OPENSSL_VERSION_NUMBER < 0x010100000) || defined(LIBRESSL_VERSION_NUMBER)) // 1.0.2 and below or LibreSSL
+// take care about openssl below 1.1.0
 #if LEGACY_OPENSSL
 // define getters and setters introduced in 1.1.0
 inline int DSA_set0_pqg(DSA *d, BIGNUM *p, BIGNUM *q, BIGNUM *g)
