@@ -12,6 +12,7 @@
 #include "I2NPProtocol.h"
 #include "Tunnel.h"
 #include "Transports.h"
+#include "NTCP2.h"
 #include "RouterContext.h"
 #include "Garlic.h"
 #include "NetDb.hpp"
@@ -25,7 +26,7 @@ namespace data
 {
 	NetDb netdb;
 
-	NetDb::NetDb (): m_IsRunning (false), m_Thread (nullptr), m_Reseeder (nullptr), m_Storage("netDb", "r", "routerInfo-", "dat"), m_FloodfillBootstrap(nullptr), m_HiddenMode(false)
+	NetDb::NetDb (): m_IsRunning (false), m_Thread (nullptr), m_Reseeder (nullptr), m_Storage("netDb", "r", "routerInfo-", "dat"), m_PersistProfiles (true), m_HiddenMode(false)
 	{
 	}
 
@@ -43,10 +44,12 @@ namespace data
 		m_Families.LoadCertificates ();
 		Load ();
 
-                uint16_t threshold; i2p::config::GetOption("reseed.threshold", threshold);
+		uint16_t threshold; i2p::config::GetOption("reseed.threshold", threshold);
 		if (m_RouterInfos.size () < threshold) // reseed if # of router less than threshold
 			Reseed ();
 
+		i2p::config::GetOption("persist.profiles", m_PersistProfiles);
+
 		m_IsRunning = true;
 		m_Thread = new std::thread (std::bind (&NetDb::Run, this));
 	}
@@ -55,8 +58,9 @@ namespace data
 	{
 		if (m_IsRunning)
 		{
-			for (auto& it: m_RouterInfos)
-				it.second->SaveProfile ();
+			if (m_PersistProfiles)
+				for (auto& it: m_RouterInfos)
+					it.second->SaveProfile ();
 			DeleteObsoleteProfiles ();
 			m_RouterInfos.clear ();
 			m_Floodfills.clear ();
@@ -98,6 +102,10 @@ namespace data
 							case eI2NPDatabaseLookup:
 								HandleDatabaseLookupMsg (msg);
 							break;
+							case eI2NPDummyMsg:
+								// plain RouterInfo from NTCP2 with flags for now
+								HandleNTCP2RouterInfoMsg (msg);
+							break;	
 							default: // WTF?
 								LogPrint (eLogError, "NetDb: unexpected message type ", (int) msg->GetTypeID ());
 								//i2p::HandleI2NPMessage (msg);
@@ -130,18 +138,17 @@ namespace data
 					lastDestinationCleanup = ts;
 				}
 
-				if (ts - lastPublish >= NETDB_PUBLISH_INTERVAL && !m_HiddenMode) // publish
+				if (ts - lastPublish >= NETDB_PUBLISH_INTERVAL) // update timestamp and publish 
 				{
-					Publish ();
+					i2p::context.UpdateTimestamp (ts);
+					if (!m_HiddenMode) Publish ();
 					lastPublish = ts;
 				}
 				if (ts - lastExploratory >= 30) // exploratory every 30 seconds
 				{
 					auto numRouters = m_RouterInfos.size ();
-					if (numRouters == 0)
-					{
-                                                throw std::runtime_error("No known routers, reseed seems to be totally failed");
-					}
+					if (!numRouters)
+                    	throw std::runtime_error("No known routers, reseed seems to be totally failed");
 					else // we have peers now
 						m_FloodfillBootstrap = nullptr;
 					if (numRouters < 2500 || ts - lastExploratory >= 90)
@@ -163,22 +170,38 @@ namespace data
 		}
 	}
 
+	void NetDb::SetHidden(bool hide) 
+	{
+    	// TODO: remove reachable addresses from router info
+    	m_HiddenMode = hide;
+  	}
+
 	bool NetDb::AddRouterInfo (const uint8_t * buf, int len)
+	{
+		bool updated;
+		AddRouterInfo (buf, len, updated);
+		return updated;	
+	}
+
+	std::shared_ptr<const RouterInfo> NetDb::AddRouterInfo (const uint8_t * buf, int len, bool& updated)
 	{
 		IdentityEx identity;
 		if (identity.FromBuffer (buf, len))
-			return AddRouterInfo (identity.GetIdentHash (), buf, len);
-		return false;
+			return AddRouterInfo (identity.GetIdentHash (), buf, len, updated);
+		updated = false;
+		return nullptr;
 	}
 
-  void NetDb::SetHidden(bool hide) {
-    // TODO: remove reachable addresses from router info
-    m_HiddenMode = hide;
-  }
-
 	bool NetDb::AddRouterInfo (const IdentHash& ident, const uint8_t * buf, int len)
 	{
-		bool updated = true;
+		bool updated;
+		AddRouterInfo (ident, buf, len, updated);
+		return updated;	
+	}
+
+	std::shared_ptr<const RouterInfo> NetDb::AddRouterInfo (const IdentHash& ident, const uint8_t * buf, int len, bool& updated)
+	{
+		updated = true;
 		auto r = FindRouter (ident);
 		if (r)
 		{
@@ -224,50 +247,56 @@ namespace data
 		}
 		// take care about requested destination
 		m_Requests.RequestComplete (ident, r);
-		return updated;
+		return r;
 	}
 
-	bool NetDb::AddLeaseSet (const IdentHash& ident, const uint8_t * buf, int len,
-		std::shared_ptr<i2p::tunnel::InboundTunnel> from)
+	bool NetDb::AddLeaseSet (const IdentHash& ident, const uint8_t * buf, int len)
 	{
 		std::unique_lock<std::mutex> lock(m_LeaseSetsMutex);
 		bool updated = false;
-		if (!from) // unsolicited LS must be received directly
+		auto it = m_LeaseSets.find(ident);
+		if (it != m_LeaseSets.end () && it->second->GetStoreType () == i2p::data::NETDB_STORE_TYPE_LEASESET)
 		{
-			auto it = m_LeaseSets.find(ident);
-			if (it != m_LeaseSets.end ())
+			// we update only is existing LeaseSet is not LeaseSet2
+			uint64_t expires;
+			if(LeaseSetBufferValidate(buf, len, expires))
 			{
-				uint64_t expires;
-				if(LeaseSetBufferValidate(buf, len, expires))
+				if(it->second->GetExpirationTime() < expires)
 				{
-					if(it->second->GetExpirationTime() < expires)
-					{
-						it->second->Update (buf, len, false); // signature is verified already
-						LogPrint (eLogInfo, "NetDb: LeaseSet updated: ", ident.ToBase32());
-						updated = true;
-					}
-					else
-						LogPrint(eLogDebug, "NetDb: LeaseSet is older: ", ident.ToBase32());
-				}
-				else
-					LogPrint(eLogError, "NetDb: LeaseSet is invalid: ", ident.ToBase32());
-			}
-			else
-			{
-				auto leaseSet = std::make_shared<LeaseSet> (buf, len, false); // we don't need leases in netdb
-				if (leaseSet->IsValid ())
-				{
-					LogPrint (eLogInfo, "NetDb: LeaseSet added: ", ident.ToBase32());
-					m_LeaseSets[ident] = leaseSet;
+					it->second->Update (buf, len, false); // signature is verified already
+					LogPrint (eLogInfo, "NetDb: LeaseSet updated: ", ident.ToBase32());
 					updated = true;
 				}
 				else
-					LogPrint (eLogError, "NetDb: new LeaseSet validation failed: ", ident.ToBase32());
+					LogPrint(eLogDebug, "NetDb: LeaseSet is older: ", ident.ToBase32());
 			}
+			else
+				LogPrint(eLogError, "NetDb: LeaseSet is invalid: ", ident.ToBase32());
+		}
+		else
+		{
+			auto leaseSet = std::make_shared<LeaseSet> (buf, len, false); // we don't need leases in netdb
+			if (leaseSet->IsValid ())
+			{
+				LogPrint (eLogInfo, "NetDb: LeaseSet added: ", ident.ToBase32());
+				m_LeaseSets[ident] = leaseSet;
+				updated = true;
+			}
+			else
+				LogPrint (eLogError, "NetDb: new LeaseSet validation failed: ", ident.ToBase32());
 		}
 		return updated;
 	}
 
+	bool NetDb::AddLeaseSet2 (const IdentHash& ident, const uint8_t * buf, int len, uint8_t storeType)
+	{
+		std::unique_lock<std::mutex> lock(m_LeaseSetsMutex);
+		// always new LS2 for now. TODO: implement update
+		auto leaseSet = std::make_shared<LeaseSet2> (storeType, buf, len, false); // we don't need leases in netdb
+		m_LeaseSets[ident] = leaseSet;
+		return true; 
+	}
+
 	std::shared_ptr<RouterInfo> NetDb::FindRouter (const IdentHash& ident) const
 	{
 		std::unique_lock<std::mutex> l(m_RouterInfosMutex);
@@ -519,7 +548,7 @@ namespace data
 				{
 					if (it->second->IsUnreachable ())
 					{
-						it->second->SaveProfile ();
+						if (m_PersistProfiles) it->second->SaveProfile ();
 						it = m_RouterInfos.erase (it);
 						continue;
 					}
@@ -571,6 +600,17 @@ namespace data
 		transports.SendMessage (from, dest->CreateRequestMessage (nullptr, nullptr));
 	}
 
+	void NetDb::HandleNTCP2RouterInfoMsg (std::shared_ptr<const I2NPMessage> m)
+	{
+		uint8_t flood = m->GetPayload ()[0] & NTCP2_ROUTER_INFO_FLAG_REQUEST_FLOOD;
+		bool updated;
+		auto ri = AddRouterInfo (m->GetPayload () + 1, m->GetPayloadLength () - 1, updated); // without flags
+		if (flood && updated && context.IsFloodfill () && ri)
+		{
+			auto floodMsg = CreateDatabaseStoreMsg (ri, 0); // replyToken = 0
+			Flood (ri->GetIdentHash (), floodMsg);
+		}
+	}
 
 	void NetDb::HandleDatabaseStoreMsg (std::shared_ptr<const I2NPMessage> m)
 	{
@@ -611,12 +651,24 @@ namespace data
 		size_t payloadOffset = offset;
 
 		bool updated = false;
-		if (buf[DATABASE_STORE_TYPE_OFFSET]) // type
+		uint8_t storeType = buf[DATABASE_STORE_TYPE_OFFSET];	
+		if (storeType) // LeaseSet or LeaseSet2
 		{
-			LogPrint (eLogDebug, "NetDb: store request: LeaseSet for ", ident.ToBase32());
-			updated = AddLeaseSet (ident, buf + offset, len - offset, m->from);
+			if (!m->from) // unsolicited LS must be received directly
+			{	
+				if (storeType == NETDB_STORE_TYPE_LEASESET) // 1
+				{
+					LogPrint (eLogDebug, "NetDb: store request: LeaseSet for ", ident.ToBase32());
+					updated = AddLeaseSet (ident, buf + offset, len - offset);
+				}
+				else // all others are considered as LeaseSet2 
+				{
+					LogPrint (eLogDebug, "NetDb: store request: LeaseSet2 of type ", storeType, " for ", ident.ToBase32());
+					updated = AddLeaseSet2 (ident, buf + offset, len - offset, storeType);
+				}
+			}	
 		}
-		else
+		else // RouterInfo
 		{
 			LogPrint (eLogDebug, "NetDb: store request: RouterInfo");
 			size_t size = bufbe16toh (buf + offset);
@@ -650,22 +702,7 @@ namespace data
 			{
 				memcpy (payload + DATABASE_STORE_HEADER_SIZE, buf + payloadOffset, msgLen);
 				floodMsg->FillI2NPMessageHeader (eI2NPDatabaseStore);
-				std::set<IdentHash> excluded;
-				excluded.insert (i2p::context.GetIdentHash ()); // don't flood to itself
-				excluded.insert (ident); // don't flood back
-				for (int i = 0; i < 3; i++)
-				{
-					auto floodfill = GetClosestFloodfill (ident, excluded);
-					if (floodfill)
-					{
-						auto h = floodfill->GetIdentHash();
-						LogPrint(eLogDebug, "NetDb: Flood lease set for ", ident.ToBase32(), " to ", h.ToBase64());
-						transports.SendMessage (h, CopyI2NPMessage(floodMsg));
-						excluded.insert (h);
-					}
-					else
-						break;
-				}
+				Flood (ident, floodMsg);
 			}
 			else
 				LogPrint (eLogError, "NetDb: Database store message is too long ", floodMsg->len);
@@ -734,7 +771,7 @@ namespace data
 					m_Requests.RequestComplete (ident, nullptr);
 			}
 			else
-				// no more requests for detination possible. delete it
+				// no more requests for destination possible. delete it
 				m_Requests.RequestComplete (ident, nullptr);
 		}
 		else if(!m_FloodfillBootstrap)
@@ -966,6 +1003,26 @@ namespace data
 		}
 	}
 
+	void NetDb::Flood (const IdentHash& ident, std::shared_ptr<I2NPMessage> floodMsg)
+	{
+		std::set<IdentHash> excluded;
+		excluded.insert (i2p::context.GetIdentHash ()); // don't flood to itself
+		excluded.insert (ident); // don't flood back
+		for (int i = 0; i < 3; i++)
+		{
+			auto floodfill = GetClosestFloodfill (ident, excluded);
+			if (floodfill)
+			{
+				auto h = floodfill->GetIdentHash();
+				LogPrint(eLogDebug, "NetDb: Flood lease set for ", ident.ToBase32(), " to ", h.ToBase64());
+				transports.SendMessage (h, CopyI2NPMessage(floodMsg));
+				excluded.insert (h);
+			}
+			else
+				break;
+		}
+	}
+
 	std::shared_ptr<const RouterInfo> NetDb::GetRandomRouter () const
 	{
 		return GetRandomRouter (
@@ -1159,9 +1216,9 @@ namespace data
 		auto ts = i2p::util::GetMillisecondsSinceEpoch ();
 		for (auto it = m_LeaseSets.begin (); it != m_LeaseSets.end ();)
 		{
-			if (ts > it->second->GetExpirationTime () - LEASE_ENDDATE_THRESHOLD)
+			if (!it->second->IsValid () || ts > it->second->GetExpirationTime () - LEASE_ENDDATE_THRESHOLD)
 			{
-				LogPrint (eLogInfo, "NetDb: LeaseSet ", it->second->GetIdentHash ().ToBase64 (), " expired");
+				LogPrint (eLogInfo, "NetDb: LeaseSet ", it->second->GetIdentHash ().ToBase64 (), " expired or invalid");
 				it = m_LeaseSets.erase (it);
 			}
 			else
