@@ -6,7 +6,6 @@
 #include <openssl/dsa.h>
 #include <openssl/ec.h>
 #include <openssl/ecdsa.h>
-#include <openssl/rsa.h>
 #include <openssl/evp.h>
 #include "Crypto.h"
 #include "Ed25519.h"
@@ -25,6 +24,7 @@ namespace crypto
 			virtual size_t GetPublicKeyLen () const = 0;
 			virtual size_t GetSignatureLen () const = 0;
 			virtual size_t GetPrivateKeyLen () const { return GetSignatureLen ()/2; };
+			virtual void SetPublicKey (const uint8_t * signingKey) = 0;
 	};
 
 	class Signer
@@ -42,9 +42,13 @@ namespace crypto
 	{
 		public:
 
-			DSAVerifier (const uint8_t * signingKey)
+			DSAVerifier ()
 			{
 				m_PublicKey = CreateDSA ();
+			}
+			
+			void SetPublicKey (const uint8_t * signingKey)
+			{	
 				DSA_set0_key (m_PublicKey, BN_bin2bn (signingKey, DSA_PUBLIC_KEY_LENGTH, NULL), NULL);
 			}
 
@@ -149,14 +153,19 @@ namespace crypto
 		enum { hashLen = 64 };
 	};
 
+	// EcDSA
 	template<typename Hash, int curve, size_t keyLen>
 	class ECDSAVerifier: public Verifier
 	{
 		public:
 
-			ECDSAVerifier (const uint8_t * signingKey)
+			ECDSAVerifier ()
 			{
 				m_PublicKey = EC_KEY_new_by_curve_name (curve);
+			}
+			
+			void SetPublicKey (const uint8_t * signingKey)
+			{	
 				BIGNUM * x = BN_bin2bn (signingKey, keyLen/2, NULL);
 				BIGNUM * y = BN_bin2bn (signingKey + keyLen/2, keyLen/2, NULL);
 				EC_KEY_set_public_key_affine_coordinates (m_PublicKey, x, y);
@@ -269,104 +278,16 @@ namespace crypto
 		CreateECDSARandomKeys (NID_secp521r1, ECDSAP521_KEY_LENGTH, signingPrivateKey, signingPublicKey);
 	}
 
-// RSA
-	template<typename Hash, int type, size_t keyLen>
-	class RSAVerifier: public Verifier
-	{
-		public:
-
-			RSAVerifier (const uint8_t * signingKey)
-			{
-				m_PublicKey = RSA_new ();
-				RSA_set0_key (m_PublicKey, BN_bin2bn (signingKey, keyLen, NULL) /* n */ , BN_dup (GetRSAE ()) /* d */, NULL);
-			}
-
-			~RSAVerifier ()
-			{
-				RSA_free (m_PublicKey);
-			}
-
-			bool Verify (const uint8_t * buf, size_t len, const uint8_t * signature) const
-			{
-				uint8_t digest[Hash::hashLen];
-				Hash::CalculateHash (buf, len, digest);
-				return RSA_verify (type, digest, Hash::hashLen, signature, GetSignatureLen (), m_PublicKey);
-			}
-			size_t GetPublicKeyLen () const { return keyLen; }
-			size_t GetSignatureLen () const { return keyLen; }
-			size_t GetPrivateKeyLen () const { return GetSignatureLen ()*2; };
-
-		private:
-
-			RSA * m_PublicKey;
-	};
-
-
-	template<typename Hash, int type, size_t keyLen>
-	class RSASigner: public Signer
-	{
-		public:
-
-			RSASigner (const uint8_t * signingPrivateKey)
-			{
-				m_PrivateKey = RSA_new ();
-				RSA_set0_key (m_PrivateKey, BN_bin2bn (signingPrivateKey, keyLen, NULL), /* n */
-					BN_dup (GetRSAE ()) /* e */, BN_bin2bn (signingPrivateKey + keyLen, keyLen, NULL) /* d */);
-			}
-
-			~RSASigner ()
-			{
-				RSA_free (m_PrivateKey);
-			}
-
-			void Sign (const uint8_t * buf, int len, uint8_t * signature) const
-			{
-				uint8_t digest[Hash::hashLen];
-				Hash::CalculateHash (buf, len, digest);
-				unsigned int signatureLen = keyLen;
-				RSA_sign (type, digest, Hash::hashLen, signature, &signatureLen, m_PrivateKey);
-			}
-
-		private:
-
-			RSA * m_PrivateKey;
-	};
-
-	inline void CreateRSARandomKeys (size_t publicKeyLen, uint8_t * signingPrivateKey, uint8_t * signingPublicKey)
-	{
-		RSA * rsa = RSA_new ();
-		BIGNUM * e = BN_dup (GetRSAE ()); // make it non-const
-		RSA_generate_key_ex (rsa, publicKeyLen*8, e, NULL);
-		const BIGNUM * n, * d, * e1;
-		RSA_get0_key (rsa, &n, &e1, &d);
-		bn2buf (n, signingPrivateKey, publicKeyLen);
-		bn2buf (d, signingPrivateKey + publicKeyLen, publicKeyLen);
-		bn2buf (n, signingPublicKey, publicKeyLen);
-		BN_free (e); // this e is not assigned to rsa->e
-		RSA_free (rsa);
-	}
-
-//  RSA_SHA256_2048
-	const size_t RSASHA2562048_KEY_LENGTH = 256;
-	typedef RSAVerifier<SHA256Hash, NID_sha256, RSASHA2562048_KEY_LENGTH> RSASHA2562048Verifier;
-	typedef RSASigner<SHA256Hash, NID_sha256, RSASHA2562048_KEY_LENGTH> RSASHA2562048Signer;
-
-//  RSA_SHA384_3072
-	const size_t RSASHA3843072_KEY_LENGTH = 384;
-	typedef RSAVerifier<SHA384Hash, NID_sha384, RSASHA3843072_KEY_LENGTH> RSASHA3843072Verifier;
-	typedef RSASigner<SHA384Hash, NID_sha384, RSASHA3843072_KEY_LENGTH> RSASHA3843072Signer;
-
-//  RSA_SHA512_4096
-	const size_t RSASHA5124096_KEY_LENGTH = 512;
-	typedef RSAVerifier<SHA512Hash, NID_sha512, RSASHA5124096_KEY_LENGTH> RSASHA5124096Verifier;
-	typedef RSASigner<SHA512Hash, NID_sha512, RSASHA5124096_KEY_LENGTH> RSASHA5124096Signer;
 
 	// EdDSA
 	class EDDSA25519Verifier: public Verifier
 	{
 		public:
 
-			EDDSA25519Verifier (const uint8_t * signingKey);
+			EDDSA25519Verifier ();
+			void SetPublicKey (const uint8_t * signingKey);
+			~EDDSA25519Verifier ();
+			
 			bool Verify (const uint8_t * buf, size_t len, const uint8_t * signature) const;
 
 			size_t GetPublicKeyLen () const { return EDDSA25519_PUBLIC_KEY_LENGTH; };
@@ -374,30 +295,72 @@ namespace crypto
 
 		private:
 
+#if OPENSSL_EDDSA
+			EVP_PKEY * m_Pkey;
+			EVP_MD_CTX * m_MDCtx;
+#else			
 			EDDSAPoint m_PublicKey;
 			uint8_t m_PublicKeyEncoded[EDDSA25519_PUBLIC_KEY_LENGTH];
+#endif			
+	};
+
+	class EDDSA25519SignerCompat: public Signer
+	{
+		public:
+
+			EDDSA25519SignerCompat (const uint8_t * signingPrivateKey, const uint8_t * signingPublicKey = nullptr);
+			// we pass signingPublicKey to check if it matches private key
+			~EDDSA25519SignerCompat ();
+			
+			void Sign (const uint8_t * buf, int len, uint8_t * signature) const;
+			const uint8_t * GetPublicKey () const { return m_PublicKeyEncoded; }; // for keys creation
+
+		private:
+		
+			uint8_t m_ExpandedPrivateKey[64];
+			uint8_t m_PublicKeyEncoded[EDDSA25519_PUBLIC_KEY_LENGTH];
 	};
 
+#if OPENSSL_EDDSA	
 	class EDDSA25519Signer: public Signer
 	{
 		public:
 
 			EDDSA25519Signer (const uint8_t * signingPrivateKey, const uint8_t * signingPublicKey = nullptr);
 			// we pass signingPublicKey to check if it matches private key
+			~EDDSA25519Signer ();
+			
 			void Sign (const uint8_t * buf, int len, uint8_t * signature) const;
-			const uint8_t * GetPublicKey () const { return m_PublicKeyEncoded; };
 
 		private:
-
-			uint8_t m_ExpandedPrivateKey[64];
-			uint8_t m_PublicKeyEncoded[EDDSA25519_PUBLIC_KEY_LENGTH];
+			EVP_PKEY * m_Pkey;
+			EVP_MD_CTX * m_MDCtx;	
+			EDDSA25519SignerCompat * m_Fallback;
 	};
+#else
 
+	typedef EDDSA25519SignerCompat EDDSA25519Signer;
+	
+#endif	
+	
 	inline void CreateEDDSA25519RandomKeys (uint8_t * signingPrivateKey, uint8_t * signingPublicKey)
 	{
+#if OPENSSL_EDDSA	
+		EVP_PKEY *pkey = NULL;
+		EVP_PKEY_CTX *pctx = EVP_PKEY_CTX_new_id (EVP_PKEY_ED25519, NULL);
+		EVP_PKEY_keygen_init (pctx);
+		EVP_PKEY_keygen (pctx, &pkey);
+		EVP_PKEY_CTX_free (pctx);
+		size_t len = EDDSA25519_PUBLIC_KEY_LENGTH;
+		EVP_PKEY_get_raw_public_key (pkey, signingPublicKey, &len);
+		len = EDDSA25519_PRIVATE_KEY_LENGTH;
+		EVP_PKEY_get_raw_private_key (pkey, signingPrivateKey, &len);
+		EVP_PKEY_free (pkey);	
+#else		
 		RAND_bytes (signingPrivateKey, EDDSA25519_PRIVATE_KEY_LENGTH);
 		EDDSA25519Signer signer (signingPrivateKey);
 		memcpy (signingPublicKey, signer.GetPublicKey (), EDDSA25519_PUBLIC_KEY_LENGTH);
+#endif		
 	}
 
 
@@ -433,15 +396,22 @@ namespace crypto
 
 			enum { keyLen = Hash::hashLen };
 
-			GOSTR3410Verifier (GOSTR3410ParamSet paramSet, const uint8_t * signingKey):
-				m_ParamSet (paramSet)
+			GOSTR3410Verifier (GOSTR3410ParamSet paramSet):
+				m_ParamSet (paramSet), m_PublicKey (nullptr)
+			{
+			}		
+
+			void SetPublicKey (const uint8_t * signingKey)	
 			{
 				BIGNUM * x = BN_bin2bn (signingKey, GetPublicKeyLen ()/2, NULL);
 				BIGNUM * y = BN_bin2bn (signingKey + GetPublicKeyLen ()/2, GetPublicKeyLen ()/2, NULL);
 				m_PublicKey = GetGOSTR3410Curve (m_ParamSet)->CreatePoint (x, y);
 				BN_free (x); BN_free (y);
 			}
-			~GOSTR3410Verifier () { EC_POINT_free (m_PublicKey); }
+			~GOSTR3410Verifier () 
+			{ 
+				if (m_PublicKey) EC_POINT_free (m_PublicKey); 
+			}
 
 			bool Verify (const uint8_t * buf, size_t len, const uint8_t * signature) const
 			{
