@@ -35,8 +35,11 @@ namespace transport
 
 	void DHKeysPairSupplier::Stop ()
 	{
-		m_IsRunning = false;
-		m_Acquired.notify_one ();
+		{
+			std::unique_lock<std::mutex> l(m_AcquiredMutex);
+			m_IsRunning = false;
+			m_Acquired.notify_one ();
+		}
 		if (m_Thread)
 		{
 			m_Thread->join ();
@@ -50,19 +53,20 @@ namespace transport
 		while (m_IsRunning)
 		{
 			int num, total = 0;
-			while ((num = m_QueueSize - (int)m_Queue.size ()) > 0 && total < 20)
+			while ((num = m_QueueSize - (int)m_Queue.size ()) > 0 && total < 10)
 			{
 				CreateDHKeysPairs (num);
 				total += num;
 			}
-			if (total >= 20)
+			if (total >= 10)
 			{
 				LogPrint (eLogWarning, "Transports: ", total, " DH keys generated at the time");
 				std::this_thread::sleep_for (std::chrono::seconds(1)); // take a break
 			}
 			else
 			{
-				std::unique_lock<std::mutex>	l(m_AcquiredMutex);
+				std::unique_lock<std::mutex> l(m_AcquiredMutex);
+				if (!m_IsRunning) break;
 				m_Acquired.wait (l); // wait for element gets acquired
 			}
 		}
@@ -117,7 +121,8 @@ namespace transport
 	Transports::Transports ():
 		m_IsOnline (true), m_IsRunning (false), m_IsNAT (true), m_Thread (nullptr), m_Service (nullptr),
 		m_Work (nullptr), m_PeerCleanupTimer (nullptr), m_PeerTestTimer (nullptr),
-		m_NTCPServer (nullptr), m_SSUServer (nullptr), m_DHKeysPairSupplier (5), // 5 pre-generated keys
+		m_NTCPServer (nullptr), m_SSUServer (nullptr), m_NTCP2Server (nullptr),
+		m_DHKeysPairSupplier (5), // 5 pre-generated keys
 		m_TotalSentBytes(0), m_TotalReceivedBytes(0), m_TotalTransitTransmittedBytes (0),
 		m_InBandwidth (0), m_OutBandwidth (0), m_TransitBandwidth(0),
 		m_LastInBandwidthUpdateBytes (0), m_LastOutBandwidthUpdateBytes (0),
@@ -191,6 +196,13 @@ namespace transport
 				LogPrint(eLogError, "Transports: invalid NTCP proxy url ", ntcpproxy);
 			return;
 		}
+		// create NTCP2. TODO: move to acceptor	
+		bool ntcp2;  i2p::config::GetOption("ntcp2.enabled", ntcp2);
+		if (ntcp2)
+		{
+			m_NTCP2Server = new NTCP2Server ();
+			m_NTCP2Server->Start ();
+		}	
 
 		// create acceptors
 		auto& addresses = context.GetRouterInfo ().GetAddresses ();
@@ -262,6 +274,13 @@ namespace transport
 			m_NTCPServer = nullptr;
 		}
 
+		if (m_NTCP2Server)
+		{
+			m_NTCP2Server->Stop ();
+			delete m_NTCP2Server;
+			m_NTCP2Server = nullptr;
+		}
+
 		m_DHKeysPairSupplier.Stop ();
 		m_IsRunning = false;
 		if (m_Service) m_Service->stop ();
@@ -386,9 +405,24 @@ namespace transport
 	{
 		if (peer.router) // we have RI already
 		{
-			if (!peer.numAttempts) // NTCP
+			if (!peer.numAttempts) // NTCP2 
 			{
 				peer.numAttempts++;
+				if (m_NTCP2Server) // we support NTCP2
+				{
+					// NTCP2 have priority over NTCP
+					auto address = peer.router->GetNTCP2Address (true, !context.SupportsV6 ()); // published only
+					if (address)
+					{
+						auto s = std::make_shared<NTCP2Session> (*m_NTCP2Server, peer.router);
+						m_NTCP2Server->Connect (address->host, address->port, s);
+						return true;
+					}	
+				}	
+			}
+			if (peer.numAttempts == 1) // NTCP1
+			{
+				peer.numAttempts++;	
 				auto address = peer.router->GetNTCPAddress (!context.SupportsV6 ());
 				if (address && m_NTCPServer)
 				{
@@ -425,28 +459,11 @@ namespace transport
 							}
 						}
 					}
-					else // we don't have address
-					{
-						if (address->addressString.length () > 0) // trying to resolve
-						{
-							if(m_NTCPServer->UsingProxy())
-							{
-								auto s = std::make_shared<NTCPSession> (*m_NTCPServer, peer.router);
-								m_NTCPServer->ConnectWithProxy(address->addressString, address->port, NTCPServer::eHostname, s);
-							}
-							else
-							{
-								LogPrint (eLogDebug, "Transports: Resolving NTCP ", address->addressString);
-								NTCPResolve (address->addressString, ident);
-							}
-							return true;
-						}
-					}
 				}
 				else
 					LogPrint (eLogDebug, "Transports: NTCP address is not present for ", i2p::data::GetIdentHashAbbreviation (ident), ", trying SSU");
 			}
-			if (peer.numAttempts == 1)// SSU
+			if (peer.numAttempts == 2)// SSU
 			{
 				peer.numAttempts++;
 				if (m_SSUServer && peer.router->IsSSU (!context.SupportsV6 ()))
@@ -463,15 +480,6 @@ namespace transport
 						m_SSUServer->CreateSession (peer.router, address->host, address->port);
 						return true;
 					}
-					else // we don't have address
-					{
-						if (address->addressString.length () > 0) // trying to resolve
-						{
-							LogPrint (eLogDebug, "Transports: Resolving SSU ", address->addressString);
-							SSUResolve (address->addressString, ident);
-							return true;
-						}
-					}
 				}
 			}
 			LogPrint (eLogInfo, "Transports: No NTCP or SSU addresses available");
@@ -513,92 +521,7 @@ namespace transport
 			}
 		}
 	}
-
-	void Transports::NTCPResolve (const std::string& addr, const i2p::data::IdentHash& ident)
-	{
-		auto resolver = std::make_shared<boost::asio::ip::tcp::resolver>(*m_Service);
-		resolver->async_resolve (boost::asio::ip::tcp::resolver::query (addr, ""),
-			std::bind (&Transports::HandleNTCPResolve, this,
-				std::placeholders::_1, std::placeholders::_2, ident, resolver));
-	}
-
-	void Transports::HandleNTCPResolve (const boost::system::error_code& ecode, boost::asio::ip::tcp::resolver::iterator it,
-		i2p::data::IdentHash ident, std::shared_ptr<boost::asio::ip::tcp::resolver> resolver)
-	{
-		auto it1 = m_Peers.find (ident);
-		if (it1 != m_Peers.end ())
-		{
-			auto& peer = it1->second;
-			if (!ecode && peer.router)
-			{
-				while (it != boost::asio::ip::tcp::resolver::iterator())
-				{
-					auto address = (*it).endpoint ().address ();
-					LogPrint (eLogDebug, "Transports: ", (*it).host_name (), " has been resolved to ", address);
-					if (address.is_v4 () || context.SupportsV6 ())
-					{
-						auto addr = peer.router->GetNTCPAddress (); // TODO: take one we requested
-						if (addr)
-						{
-							auto s = std::make_shared<NTCPSession> (*m_NTCPServer, peer.router);
-							m_NTCPServer->Connect (address, addr->port, s);
-							return;
-						}
-						break;
-					}
-					else
-						LogPrint (eLogInfo, "Transports: NTCP ", address, " is not supported");
-					it++;
-				}
-			}
-			LogPrint (eLogError, "Transports: Unable to resolve NTCP address: ", ecode.message ());
-			std::unique_lock<std::mutex> l(m_PeersMutex);
-			m_Peers.erase (it1);
-		}
-	}
-
-	void Transports::SSUResolve (const std::string& addr, const i2p::data::IdentHash& ident)
-	{
-		auto resolver = std::make_shared<boost::asio::ip::tcp::resolver>(*m_Service);
-		resolver->async_resolve (boost::asio::ip::tcp::resolver::query (addr, ""),
-			std::bind (&Transports::HandleSSUResolve, this,
-				std::placeholders::_1, std::placeholders::_2, ident, resolver));
-	}
-
-	void Transports::HandleSSUResolve (const boost::system::error_code& ecode, boost::asio::ip::tcp::resolver::iterator it,
-		i2p::data::IdentHash ident, std::shared_ptr<boost::asio::ip::tcp::resolver> resolver)
-	{
-		auto it1 = m_Peers.find (ident);
-		if (it1 != m_Peers.end ())
-		{
-			auto& peer = it1->second;
-			if (!ecode && peer.router)
-			{
-				while (it != boost::asio::ip::tcp::resolver::iterator())
-				{
-					auto address = (*it).endpoint ().address ();
-					LogPrint (eLogDebug, "Transports: ", (*it).host_name (), " has been resolved to ", address);
-					if (address.is_v4 () || context.SupportsV6 ())
-					{
-						auto addr = peer.router->GetSSUAddress (); // TODO: take one we requested
-						if (addr)
-						{
-							m_SSUServer->CreateSession (peer.router, address, addr->port);
-							return;
-						}
-						break;
-					}
-					else
-						LogPrint (eLogInfo, "Transports: SSU ", address, " is not supported");
-					it++;
-				}
-			}
-			LogPrint (eLogError, "Transports: Unable to resolve SSU address: ", ecode.message ());
-			std::unique_lock<std::mutex> l(m_PeersMutex);
-			m_Peers.erase (it1);
-		}
-	}
-
+	
 	void Transports::CloseSession (std::shared_ptr<const i2p::data::RouterInfo> router)
 	{
 		if (!router) return;
@@ -709,7 +632,7 @@ namespace transport
 						sendDatabaseStore = false; // we have it in the list already
 				}
 				if (sendDatabaseStore)
-					session->SendI2NPMessages ({ CreateDatabaseStoreMsg () });
+					session->SendLocalRouterInfo ();
 				else
 					session->SetTerminationTimeout (10); // most likely it's publishing, no follow-up messages expected, set timeout to 10 seconds
 				it->second.sessions.push_back (session);
@@ -783,7 +706,6 @@ namespace transport
 					if (profile)
 					{
 						profile->TunnelNonReplied();
-						profile->Save(it->first);
 					}
 					std::unique_lock<std::mutex>	l(m_PeersMutex);
 					it = m_Peers.erase (it);
