@@ -14,7 +14,7 @@ namespace i2p
 namespace client
 {
 	class I2PServiceHandler;
-	class I2PService : std::enable_shared_from_this<I2PService>
+	class I2PService : public std::enable_shared_from_this<I2PService>
 	{
 		public:
 			typedef std::function<void(const boost::system::error_code &)> ReadyCallback;
@@ -42,11 +42,11 @@ namespace client
 
 			inline std::shared_ptr<ClientDestination> GetLocalDestination () { return m_LocalDestination; }
 			inline std::shared_ptr<const ClientDestination> GetLocalDestination () const  { return m_LocalDestination; }
-			inline void SetLocalDestination (std::shared_ptr<ClientDestination> dest) 
-			{ 
+			inline void SetLocalDestination (std::shared_ptr<ClientDestination> dest)
+			{
 				if (m_LocalDestination) m_LocalDestination->Release ();
 				if (dest) dest->Acquire ();
-				m_LocalDestination = dest; 
+				m_LocalDestination = dest;
 			}
 			void CreateStream (StreamRequestComplete streamRequestComplete, const std::string& dest, int port = 0);
 			void CreateStream(StreamRequestComplete complete, const i2p::data::IdentHash & ident, int port);
@@ -67,8 +67,11 @@ namespace client
 			std::mutex m_HandlersMutex;
 			std::vector<std::pair<ReadyCallback, uint32_t> > m_ReadyCallbacks;
 			boost::asio::deadline_timer m_ReadyTimer;
+            bool m_ReadyTimerTriggered;
 			uint32_t m_ConnectTimeout;
 
+            const size_t NEVER_TIMES_OUT = 0;
+      
 		public:
 			bool isUpdated; // transient, used during reload only
 	};
